/*
 * linotp-client.h
 *
 * Copyright (C) 2010 - 2019 KeyIdentity GmbH
 *
 * E-mail: linotp@keyidentity.com
 * Contact: www.linotp.org
 * Support: www.keyidentity.com
 *
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 */

#ifndef LINOTP_CLIENT_H
#define LINOTP_CLIENT_H

#ifdef __cplusplus
extern "C" {
#endif


#include <stdbool.h>
#include <stddef.h>
#include <stdarg.h>

#include <json.h>
#include <curl/curl.h>


#define LIB_LINOTP_VERSION	"3.1.1"



/*
 * Limits
 */

#define LINOTP_MAX_USERNAME_LEN		256
#define LINOTP_MAX_REALMNAME_LEN	256
#define LINOTP_MAX_TRANSACTIONID	32
#define LINOTP_MAX_TOKENTYPE		32
#define LINOTP_MAX_TOKENSERIAL		32
#define LINOTP_MAX_MESSAGE		512





/*
 * flags for linotp_init
 */
#define LINOTP_FLAG_NO_CERT_VERIFY		1
#define LINOTP_FLAG_NO_NAME_VERIFY		2


/**
 * Return codes used by most functions of this Library
 */
enum linotp_rc {
	LINOTP_OK = 0,				/* 0 */
	LINOTP_ERROR_PARAM_NULL,		/* 1 */
	LINOTP_ERROR_CURL_INIT,			/* 2 */
	LINOTP_ERROR_ALLOC_FAIL,		/* 3 */
	LINOTP_ERROR_CURL_SETOPT,		/* 4 */
	LINOTP_ERROR_CURL_CONN,			/* 5 */
	LINOTP_ERROR_EMPTY_ANSWER,		/* 6 */
	LINOTP_ERROR_CREATE_URL_PARAMS,		/* 7 */
	LINOTP_ERROR_JSON_PARSE,		/* 8 */
	LINOTP_ERROR_JSON_OBJ,			/* 9 */
	LINOTP_ERROR_INVAL_VER,			/* 10 */
	LINOTP_ERROR_U2F_DEV,			/* 11 */
	LINOTP_ERROR_U2F_APPID,			/* 12 */
	LINOTP_ERROR_SENDREQ,			/* 13 */
	LINOTP_ERROR_INVAL_RESPONSE,		/* 14 */
	LINOTP_ERROR_STATUS,			/* 15 */
};


struct memory_struct {
	char	*memory;
	size_t	 size;
	size_t	 alloc_size;
};


struct linotp_context {
	char			*base_url;

	/* CURL specific part */
	CURL			*curl;
	char	 		 curl_error[CURL_ERROR_SIZE];

	struct memory_struct	 curl_mem;

};


/**
 * initialize and configure linotp-client library
 */
void		 linotp_global_init();
void		 linotp_global_cleanup();

/**
 * Set debugging state
 *
 * If debugging is enabled, the library will log detailed
 * diagnostics to the log channel (not standard output),
 * including messages to and from linotp.
 *
 * *WARNING*: The log will include sensitive authentication
 * data so do not leave this on for production systems.
 */
void		 linotp_global_set_debugging(bool debug);
const char	*linotp_errstr(enum linotp_rc code);


/*
 * create and configure LinOTP session
 */

enum linotp_rc
linotp_init(struct linotp_context *ctx, const char *url, long timeout, int flags);

enum linotp_rc
linotp_cleanup(struct linotp_context *ctx);

enum linotp_rc
linotp_set_ca_file(struct linotp_context *ctx, const char *ca_file);

enum linotp_rc
linotp_set_ca_path(struct linotp_context *ctx, const char *ca_path);


/*
 * Request functions
 */

enum linotp_rc
linotp_send_request(struct linotp_context *ctx,
		    char **response,		/* output */
		    const char *url_path,
		    va_list params_list);

enum linotp_rc
linotp_call_url(struct linotp_context *ctx,
		char **response,		/* output */
		const char *url_path,
		...);

/*
 * JSON handling
 */

enum linotp_rc
linotp_json_parse_string(struct json_object **obj,
			 const char *str);

enum linotp_rc
linotp_json_get_object(struct json_object **result,
		       struct json_object *obj,
		       const char **keys);

bool linotp_json_havekey(struct json_object *obj, const char **keys);

enum linotp_rc
linotp_json_get_string(const char **result,
		       struct json_object *obj,
		       const char **keys);

enum linotp_rc
linotp_json_get_bool(bool *result,
		     struct json_object *obj,
		     const char **keys);



/*
 * LinOTP protocol
 */

bool	 linotp_proto_valid_username(const char *name);
bool	 linotp_proto_valid_realm(const char *realm);



/*
 * LinOTP API
 */


struct linotp_validate_check {
	bool	value;

	bool	details;
	char	transactionid[LINOTP_MAX_TRANSACTIONID];
	char	tokentype[LINOTP_MAX_TOKENTYPE];
	char	tokenserial[LINOTP_MAX_TOKENSERIAL];
	char	message[LINOTP_MAX_MESSAGE];
};


struct linotp_validate_status {
	bool	received;
	bool	valid;

};

enum linotp_rc
linotp_validate_check(struct linotp_context *L,
		      struct linotp_validate_check *result,
		      ...);

enum linotp_rc
linotp_validate_check_status(struct linotp_context *L,
		      struct linotp_validate_status *result,
		      ...);

/* deprecated - will at least be removed with api version 4.0 */
enum linotp_rc
linotp_validate_status(struct linotp_context *L,
		       struct linotp_validate_status *result,
		       const char *user,
		       const char *pass,
		       const char *transactionid);

#ifdef __cplusplus
}
#endif

#endif
